@@ -4,15 +4,14 @@ Common things.
 
 try:
     # Python 2.x
-    _str = unicode
+    to_string = unicode
 except NameError:
     # Pytyon 3.x
-    _str = str
-
+    to_string = str
 
 def parse_string(string, encoding):
     """Convert a byte string to unicode
 
     Also strips trailing spaces and null bytes.
     """
-    return _str(string, encoding).rstrip('\x00').rstrip(' ')
+    return to_string(string.rstrip(b'\0 '), encoding)
