@@ -12,6 +12,7 @@ from .common import parse_string
 from .ifiles import ifind
 from .fpt import FPT
 from .codepages import guess_encoding
+from .dbversions import get_dbversion_string
 
 DBFHeader = StructParser(
     'DBFHeader',
@@ -51,8 +52,6 @@ DBFField = StructParser(
      'index_field_flag',
      ])
 
-FieldValue = collections.namedtuple('Field', 'name value')
-
 
 def expand_year(year):
     """Convert 2-digit year to 4-digit year."""
@@ -64,16 +63,15 @@ def expand_year(year):
 
 
 class RecordIterator(object):
-    def __init__(self, table, record_types):
-        self._record_types = record_types
+    def __init__(self, table, record_type):
+        self._record_type = record_type
         self._table = table
 
     def __iter__(self):
-        for _, record in self._table._iter_records(self._record_types):
-            yield record
+        return self._table._iter_records(self._record_type)
  
     def __len__(self):
-        return self._table._count_records(self._record_types)
+        return self._table._count_records(self._record_type)
 
 
 class FakeMemoFile(object):
@@ -112,10 +110,9 @@ class Table(list):
 
         # Name part before .dbf is the table name
         self.name = os.path.basename(filename)
-        self.name = os.path.splitext(self.name)[0]
-        self.name = self.name.lower()
+        self.name = os.path.splitext(self.name)[0].lower()
         
-        self.deleted = []
+        self._deleted = None
         self.loaded = False
 
         if ignorecase:
@@ -124,6 +121,7 @@ class Table(list):
                 raise IOError('No such file: {!r}'.format(filename))
         else:
             self.filename = filename
+
         # Filled in by self._read_headers()
         self.memofilename = None
         self.header = None
@@ -140,6 +138,8 @@ class Table(list):
                                       self.header.month,
                                       self.header.day)
             
+        self.dbversion = get_dbversion_string(self.header.dbversion)
+
         if load:
             self.load()
         else:
@@ -150,18 +150,11 @@ class Table(list):
             return FPT(self.memofilename)
         else:
             return _FAKE_MEMOFILE
- 
+
     def load(self):
         if not self.loaded:
-            del self[:]
-            self.deleted = []
-
-            for record_type, record in self._iter_records(b' *'):
-                if record_type == b' ':
-                    self.append(record)
-                else:
-                    self.deleted.append(record)
-
+            self[:] = self._iter_records(b' ')
+            self._deleted = list(self._iter_records(b'*'))
             self.loaded = True
 
     def unload(self):
@@ -169,9 +162,16 @@ class Table(list):
         # is called by __init__() where self.loaded=False.
         # Also, unloading twice has no consequences.
         del self[:]
-        self.deleted = RecordIterator(self, b'*')
+        self._deleted = None
         self.loaded = False
 
+    @property
+    def deleted(self):
+        if self.loaded:
+            return self._deleted
+        else:
+            return RecordIterator(self, b'*')
+
     def _read_headers(self, infile):
         #
         # Todo: more checks
@@ -195,7 +195,6 @@ class Table(list):
                 break
 
             fh = DBFField.read(infile, prepend=sep)
-            # We need to fix the name and type
 
             fieldname = parse_string(fh.name, self.encoding)
             if self.lowernames:
@@ -247,10 +246,17 @@ class Table(list):
 
     def _read_record(self, infile, memofile):
         items = []  # List of Field
-        for field in self.fields:
-            value = infile.read(field.length)
-            if not self.raw:
-                value = self._field_parser.parse(field, value)
+
+        # Shortcuts for speed.
+        parse = self._field_parser.parse
+        append = items.append
+        read = infile.read
+        
+        if self.raw:
+            items = [(field.name, read(field.length)) for field in self.fields]
+        else:
+            for field in self.fields:
+                value = parse(field, read(field.length))
 
                 #
                 # Decoding memo fields requires a little more
@@ -265,52 +271,61 @@ class Table(list):
                         # Byte string
                         value = memo.data
 
-            items.append(FieldValue(name=field.name, value=value))
+                append((field.name, value))
 
         return self.recfactory(items)
         
     def _skip_record(self, infile):
+        # -1 for the record separator which was already read.
         infile.seek(self.header.recordlen - 1, 1)
 
-    def _count_records(self, record_types=b' '):
+    def _count_records(self, record_type=b' '):
         count = 0
 
         with open(self.filename, 'rb') as infile:
-            infile.seek(self.header.headerlen, 0)
-            while True:
-                sep = infile.read(1)
-                if sep in (b'\x1a', b''):
-                    # End of records.
-                    break
-                elif sep in record_types:
-                    count += 1
-                self._skip_record(infile)
-
-        return count
-
-    def _iter_records(self, record_types=b' '):
-        with open(self.filename, 'rb') as infile, \
-              self._get_memofile() as memofile:
             # Skip to first record.
             infile.seek(self.header.headerlen, 0)
+
             while True:
                 sep = infile.read(1)
-
-                if sep in (b'\x1a', b''):
+                if sep == record_type:
+                    count += 1
+                    self._skip_record(infile)
+                elif sep in (b'\x1a', b''):
                     # End of records.
                     break
-                elif sep in record_types:
-                    yield (sep, self._read_record(infile, memofile))
                 else:
                     self._skip_record(infile)
 
+        return count
+
+    def _iter_records(self, record_type=b' '):
+        with open(self.filename, 'rb') as infile, \
+             self._get_memofile() as memofile:
+            # Skip to first record.
+            infile.seek(self.header.headerlen, 0)
+
+            # Shortcuts for speed.
+            read_record = self._read_record
+            skip_record = self._skip_record
+            read = infile.read
+
+            while True:
+                sep = read(1)
+
+                if sep == record_type:
+                    yield read_record(infile, memofile)
+                elif sep in (b'\x1a', b''):
+                    # End of records.
+                    break
+                else:
+                    skip_record(infile)
+
     def __iter__(self):
         if self.loaded:
-            for record in list.__iter__(self):
-                yield record
+            return list.__iter__(self)
         else:
-            for _, record in self._iter_records():
-                yield record            
+            return self._iter_records()
 
     def __len__(self):
         if self.loaded:
@@ -323,4 +338,3 @@ class Table(list):
             return list.__repr__(self)
         else:
             return '<Unloaded DBF table {!r}>'.format(self.filename)
-
